@@ -711,36 +711,36 @@ struct coll_src {
     union {
         FILE *file;
         struct {
-            guchar *base;
-            guchar *ptr;
-            guchar *end;
-        };
-    };
+            gchar *base;
+            gchar *ptr;
+            gchar *end;
+        } mem;
+    } backend;
 };
 
-static void free_buffer(struct coll_src *source, guchar * buffer)
+static void free_buffer(struct coll_src *source, gchar * buffer)
 {
     if (source->is_file)
         g_free(buffer);
 }
 
-static guchar *read_buffer(struct coll_src *source, size_t length,
-                           gboolean is_string)
+static gchar *read_buffer(struct coll_src *source, size_t length,
+                          gboolean is_string)
 {
-    guchar *data;
+    gchar *data;
 
     if (source->is_file) {
-        data = g_new(guchar, length);
-        if (fread(data, 1, length, source->file) != length) {
+        data = g_new(gchar, length);
+        if (fread(data, 1, length, source->backend.file) != length) {
             g_free(data);
             return NULL;
         }
     } else {
-        if (source->ptr + length > source->end)
+        if (source->backend.mem.ptr + length > source->backend.mem.end)
             return NULL;
 
-        data = source->ptr;
-        source->ptr += length;
+        data = source->backend.mem.ptr;
+        source->backend.mem.ptr += length;
     }
 
     if (is_string && data[length - 1] != '\0') {
@@ -754,12 +754,12 @@ static guchar *read_buffer(struct coll_src *source, size_t length,
 static gint read_char(struct coll_src *source)
 {
     if (source->is_file)
-        return fgetc(source->file);
+        return fgetc(source->backend.file);
 
-    if (source->ptr >= source->end)
+    if (source->backend.mem.ptr >= source->backend.mem.end)
         return EOF;
 
-    return *(source->ptr++);
+    return *(source->backend.mem.ptr++);
 }
 
 static GdkPixbufDestroyNotify destroy_func(struct coll_src *source)
@@ -772,7 +772,7 @@ static gboolean check_magic(struct coll_src *source, const gchar * magic)
 {
     gboolean res;
     gint size = strlen(magic) + 1;
-    guchar *buffer = read_buffer(source, size, TRUE);
+    gchar *buffer = read_buffer(source, size, TRUE);
 
     if (buffer == NULL)
         return FALSE;
@@ -815,7 +815,7 @@ static gboolean read_pixbuf(struct coll_src *source, GdkPixbuf ** thumb)
     gboolean has_alpha;
     gint bits_per_sample, width, height, rowstride;
     gint pixel_length;
-    guchar *pixel;
+    gchar *pixel;
 
     READ_THUMB_NB(source, colorspace);
     READ_THUMB_NB(source, has_alpha);
@@ -931,22 +931,23 @@ static struct coll_src *build_source(FILE * file)
         goto no_mmap;
     }
 
-    source->base = mmap(NULL, length, PROT_READ, MAP_PRIVATE, fileno(file), 0);
-    if (source->base == MAP_FAILED) {
+    source->backend.mem.base =
+        mmap(NULL, length, PROT_READ, MAP_PRIVATE, fileno(file), 0);
+    if (source->backend.mem.base == MAP_FAILED) {
         perror("mmap");
         goto no_mmap;
     }
 
     source->is_file = FALSE;
-    source->ptr = source->base;
-    source->end = source->base + length;
+    source->backend.mem.ptr = source->backend.mem.base;
+    source->backend.mem.end = source->backend.mem.base + length;
     goto ok;
 
   no_mmap:
     fseeko(file, 0, SEEK_SET);
 
     source->is_file = TRUE;
-    source->file = file;
+    source->backend.file = file;
 
   ok:
     return source;
@@ -955,11 +956,12 @@ static struct coll_src *build_source(FILE * file)
 static void destroy_source(struct coll_src *source, gboolean ok)
 {
     if (ok == FALSE && source->is_file == FALSE)
-        if (munmap(source->base, source->end - source->base) < 0)
+        if (munmap(source->backend.mem.base,
+                   source->backend.mem.end - source->backend.mem.base) < 0)
             perror("munmap");
 }
 
-gint load_dot_gliv_from_file(FILE * file)
+gint load_dot_gliv_from_file(const gchar * filename, FILE * file)
 {
     gint count, i = 0;
     tree_item **items = NULL;
@@ -980,7 +982,7 @@ gint load_dot_gliv_from_file(FILE * file)
         goto error;
 
     if (rt)
-        create_progress_dialog(NULL, &cancel);
+        create_progress_dialog(filename, &cancel);
 
     items = g_new(tree_item *, count);
     for (i = 0; i < count; i++) {
@@ -1069,7 +1071,7 @@ gint load_dot_gliv(const gchar * filename)
             return 0;
         }
 
-        nb_inserted = load_dot_gliv_from_file(file);
+        nb_inserted = load_dot_gliv_from_file(filename, file);
         fclose(file);
     }
 
