@@ -4,6 +4,6 @@
 #include "gliv.h"
 
 gboolean start_server(void);
-gboolean connect_server(gint argc, gchar ** argv);
+gboolean connect_server(void);
 
 #endif
