@@ -4,9 +4,9 @@
 #include "gliv.h"
 #include "gliv-image.h"
 
-void after_reorder(gint new_id);
+void after_reorder(void);
 const gchar *get_image_notice(void);
-gboolean load_direction(gint dir);
+void load_direction(gint dir);
 void open_next_image(gboolean keep_next);
 GlivImage *load_first_image(void);
 void load_second_image(void);
