@@ -10,11 +10,14 @@
 #define REFRESH_STATUS     (1 << 2)
 #define APPEND_HISTORY     (1 << 3)
 #define REFRESH_BURST      (1 << 4)
+#define REFRESH_TITLE      (1 << 5)
+#define REFRESH_SCROLL     (1 << 6)
 
 void draw_current_image(void);
-gfloat *prepare_rendering(GlivImage * im);
 void render(void);
 void zoom_in(gfloat ratio);
 void refresh(gint what);
+void update_current_image_status(gboolean find_number);
+void do_later(gint priority, GSourceDummyMarshal func);
 
 #endif
