@@ -3,7 +3,7 @@
 
 #include "gliv-image.h"
 
-void transition(gfloat * old_matrix, GlivImage * im);
+void transition(GlivImage * im);
 gboolean is_in_transition(void);
 
 #endif
