@@ -408,17 +408,19 @@ void matrix_set_top_left(void)
 
 void configure_matrix(GlivImage * im)
 {
-    GlivImage *old = current_image;
+    if (im->first_image || options->keep_transfo == FALSE) {
+        GlivImage *old = current_image;
 
-    current_image = im;
-    matrix_reset();
-    if (options->maximize || options->scaledown)
-        matrix_set_max_zoom(-1, -1, TRUE);
+        current_image = im;
+        matrix_reset();
+        if (options->maximize || options->scaledown)
+            matrix_set_max_zoom(-1, -1, TRUE);
 
-    if (options->no_center)
-        matrix_set_top_left();
+        if (options->no_center)
+            matrix_set_top_left();
 
-    current_image = old;
+        current_image = old;
+    }
 }
 
 gfloat *new_matrix(void)
