@@ -41,6 +41,7 @@
 #include "callbacks.h"
 #include "transition.h"
 #include "formats.h"
+#include "windows.h"
 
 extern rt_struct *rt;
 extern options_struct *options;
@@ -70,7 +71,7 @@ static void forget_image(GlivImage ** im_ptr)
  * of the current image position and destroy the previous and
  * next images since they changed.
  */
-void after_reorder(gint new_id)
+void after_reorder(void)
 {
     if (previous_image != NULL &&
         current_image->node->prev != previous_image->node) {
@@ -82,13 +83,7 @@ void after_reorder(gint new_id)
         forget_image(&next_image);
     }
 
-    current_image->number = new_id;
-    fill_ident(current_image);
-
-    refresh(REFRESH_STATUS);
-
-    g_free(current_image->ident);
-    current_image->ident = NULL;
+    update_current_image_status(TRUE);
 }
 
 /*** Last image notice ***/
@@ -159,15 +154,22 @@ static void slide_show_advance(void);
 
 static void render_next_image(GlivImage * im)
 {
-    gfloat *old_matrix;
     gboolean first_image = current_image == NULL;
+    gboolean make_transition = options->transitions &&
+        options->trans_steps > 0 && current_image != NULL && im != NULL;
 
-    old_matrix = prepare_rendering(im);
+    if (make_transition) {
 
-    if (current_image != NULL && im != NULL)
-        transition(old_matrix, im);
+        if (options->fullscreen == FALSE)
+            goto_window(im, FALSE);
 
-    g_free(old_matrix);
+        /* Be sure to update the display now. A transition will take place ... */
+        gtk_widget_queue_draw(GTK_WIDGET(get_current_window()));
+        gdk_window_process_all_updates();
+        process_events();
+
+        transition(im);
+    }
 
     if (current_image != NULL) {
         prioritize_textures(current_image, FALSE);
@@ -178,6 +180,13 @@ static void render_next_image(GlivImage * im)
     g_object_ref(im);
     render();
 
+    if (make_transition == FALSE)
+        if (options->fullscreen == FALSE)
+            goto_window(im, FALSE);
+
+    if (options->no_center)
+        menu_set_top_left();
+
     if (im->node->next == NULL)
         image_notice(TRUE);
 
@@ -202,22 +211,96 @@ static GList *next_node(GList * node, gint dir)
     return node->prev;
 }
 
+static void show_errors(GSList * errors)
+{
+    GtkDialog *dialog;
+    GtkTextView *text;
+    GtkTextBuffer *buffer;
+    GtkScrolledWindow *scroll;
+
+    /* Create the widgets */
+
+    dialog = GTK_DIALOG(gtk_dialog_new_with_buttons(_("Loading errors"),
+                                                    get_current_window(),
+                                                    GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                    GTK_STOCK_OK,
+                                                    GTK_RESPONSE_NONE, NULL));
+
+    text = GTK_TEXT_VIEW(gtk_text_view_new());
+    gtk_text_view_set_wrap_mode(text, GTK_WRAP_WORD_CHAR);
+    gtk_text_view_set_editable(text, FALSE);
+    gtk_text_view_set_cursor_visible(text, FALSE);
+
+    buffer = gtk_text_view_get_buffer(text);
+    gtk_text_buffer_insert_at_cursor(buffer,
+                                     _("The following errors occurred"
+                                       " while loading the next image:\n"), -1);
+
+    while (errors) {
+        gtk_text_buffer_insert_at_cursor(buffer, errors->data, -1);
+        gtk_text_buffer_insert_at_cursor(buffer, "\n", -1);
+        errors = errors->next;
+    }
+
+    scroll = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
+    gtk_container_add(GTK_CONTAINER(scroll), GTK_WIDGET(text));
+
+    gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(scroll));
+    gtk_widget_show_all(GTK_WIDGET(scroll));
+    gtk_window_set_default_size(GTK_WINDOW(dialog), 600, 400);
+    run_modal_dialog(dialog);
+    gtk_widget_destroy(GTK_WIDGET(dialog));
+}
+
 static GlivImage *load_next_node(GList ** node, gint dir)
 {
     GlivImage *im = NULL;
     GList *next;
+    GError *error;
+    GSList *failed = NULL;
+    gint nb_errors = 0;
 
     while ((next = next_node(*node, dir)) != NULL) {
-        im = load_file(next->data);
-        if (im == NULL)
+        gchar *filename = next->data;
+
+        error = NULL;
+        im = load_file(filename, &error);
+        if (im == NULL) {
+            gchar *msg;
+
             /* Delete bad images from the list. */
             remove_from_list(next);
-        else {
+
+            if (++nb_errors < 1024) {
+                msg = error ?
+                    g_strdup(error->message) :
+                    g_strdup_printf(_("Cannot load %s"), filename);
+                failed = g_slist_prepend(failed, msg);
+            }
+            g_clear_error(&error);
+        } else {
             *node = next;
+            if (error != NULL && error->message != NULL)
+                g_printerr("%s\n", error->message);
             break;
         }
     }
 
+    if (nb_errors == 1) {
+        gchar *filename = failed->data;
+        DIALOG_MSG("%s", filename);
+    } else if (nb_errors > 1) {
+        /* Many errors */
+        GSList *ptr;
+
+        failed = g_slist_reverse(failed);
+        show_errors(failed);
+        for (ptr = failed; ptr != NULL; ptr = ptr->next)
+            g_free(ptr->data);
+
+        g_slist_free(failed);
+    }
+
     return im;
 }
 
@@ -226,6 +309,7 @@ static GlivImage *load_in_direction(gint dir)
     GlivImage *im = NULL;
     GList *node;
     gint id;
+    timestamp_t before_load = get_list_timestamp();
 
     if (get_list_length() == 2 && options->loop) {
         /*
@@ -250,7 +334,7 @@ static GlivImage *load_in_direction(gint dir)
         id = 0;
     } else {
         node = current_image->node;
-        id = current_image->number + dir;
+        id = get_image_number(current_image) + dir;
     }
 
     im = load_next_node(&node, dir);
@@ -267,6 +351,9 @@ static GlivImage *load_in_direction(gint dir)
         im->number = id;
     }
 
+    if (before_load != get_list_timestamp())
+        update_current_image_status(TRUE);
+
     return im;
 }
 
@@ -282,93 +369,78 @@ static void destroy_unused(GlivImage ** backup)
     g_object_ref(*backup);
 }
 
+static void load_next(void)
+{
+    load_direction(1);
+}
+
+static void load_prev(void)
+{
+    load_direction(-1);
+}
+
+static void stop_slide_show(void);
+
 /* Return FALSE if there are no more images. */
-gboolean load_direction(gint dir)
+void load_direction(gint dir)
 {
     GlivImage **backup;
     GlivImage **next;
     GlivImage *im;
 
-    static volatile gint next_direction;
+    /* We are not reentrant */
+    static gboolean in_load_direction = FALSE;
 
-    /*
-     * Before loading the next image in a prebuffer we let GTK process
-     * remaining events and if there is a request for the next
-     * image in the events pending then -> crash.
-     * With this boolean we process loading request only if we know
-     * all loadings are finished.
-     * IOW: we are not reentrant.
-     */
-    static gboolean load_finished = TRUE;
+    if (currently_loading() || current_image == NULL)
+        return;
 
-    if (load_finished == FALSE) {
-        /*
-         * We come from process_events() called here, so we inform
-         * that we want to load the next image.
-         */
-        next_direction = dir;
-        return TRUE;
+    if (dir == -1) {
+        backup = &next_image;
+        next = &previous_image;
+    } else if (dir == 1) {
+        backup = &previous_image;
+        next = &next_image;
+    } else
+        return;
+
+    if (in_load_direction) {
+        /* We delay the reentrance */
+        if (dir > 0)
+            do_later(G_PRIORITY_DEFAULT, load_next);
+        else
+            do_later(G_PRIORITY_DEFAULT, load_prev);
+        return;
     }
 
-    if (is_loading() || current_image == NULL)
-        return TRUE;
+    in_load_direction = TRUE;
 
-    load_finished = FALSE;
-
-    /* We process subsequent next image calls in the same loop. */
-    while (dir * dir == 1) {
-
-        if (dir < 0) {
-            /* dir == -1 */
-            backup = &next_image;
-            next = &previous_image;
-        } else {
-            /* dir == 1 */
-            backup = &previous_image;
-            next = &next_image;
-        }
-
-        /*
-         * next_direction can be changed by us when called
-         * by process_events().
-         */
-        next_direction = 0;
-
-        /* Be sure there is a next image. */
+    /* Be sure there is a next image. */
+    if (*next == NULL) {
+        *next = load_in_direction(dir);
         if (*next == NULL) {
-            *next = load_in_direction(dir);
-            if (*next == NULL) {
-                /* There is no next image to load. */
-                load_finished = TRUE;
-                image_notice(dir == 1);
-
-                return FALSE;
-            }
+            /* There is no next image to load. */
+            image_notice(dir == 1);
+            stop_slide_show();
+            goto out;
         }
-
-        destroy_unused(backup);
-        im = *next;
-        *next = NULL;
-
-        render_next_image(im);
-        g_object_unref(im);
-
-        process_events();
-
-        if (options->one_image) {
-            g_object_unref(*backup);
-            *backup = NULL;
-        } else
-            *next = load_in_direction(dir);
-
-        slide_show_direction = dir;
-
-        /* Usually next_direction == 0, and the while loop ends. */
-        dir = next_direction;
     }
 
-    load_finished = TRUE;
-    return TRUE;
+    destroy_unused(backup);
+    im = *next;
+    *next = NULL;
+
+    render_next_image(im);
+    g_object_unref(im);
+
+    if (options->one_image) {
+        g_object_unref(*backup);
+        *backup = NULL;
+    } else
+        *next = load_in_direction(dir);
+
+    slide_show_direction = dir;
+  out:
+    in_load_direction = FALSE;
 }
 
 /*
@@ -380,8 +452,9 @@ gboolean load_direction(gint dir)
 void open_next_image(gboolean keep_next)
 {
     GlivImage *next;
+    GlivImage *future_image;
 
-    if (is_loading())
+    if (currently_loading())
         return;
 
     if (keep_next) {
@@ -389,16 +462,20 @@ void open_next_image(gboolean keep_next)
     } else
         unload_images();
 
+    /* load_in_direction() would be too smart when looping with 2 images */
+    future_image = next_image;
+    next_image = NULL;
+
+    next = load_in_direction(1);
+    if (next == NULL)
+        return;
+
     if (current_image != NULL) {
         previous_image = current_image;
         g_object_ref(previous_image);
     }
 
-    next = load_in_direction(1);
-    if (next == NULL) {
-        unload_images();
-        return;
-    }
+    next_image = future_image;
 
     render_next_image(next);
     g_object_unref(next);
@@ -445,10 +522,10 @@ static void reload_current_image(void)
 {
     GlivImage *new;
 
-    if (is_loading() || current_image == NULL)
+    if (currently_loading() || current_image == NULL)
         return;
 
-    new = load_file(current_image->node->data);
+    new = load_file(current_image->node->data, NULL);
     if (new == NULL)
         return;
 
@@ -456,18 +533,16 @@ static void reload_current_image(void)
     new->number = current_image->number;
 
     g_object_unref(current_image);
+    current_image = new;
 
-    if (current_image != new) {
-        current_image = new;
-
-        prioritize_textures(current_image, TRUE);
-        refresh(REFRESH_NOW);
-    }
+    prioritize_textures(current_image, TRUE);
+    refresh(REFRESH_NOW);
+    update_current_image_status(FALSE);
 }
 
 void reload_images(void)
 {
-    if (is_loading())
+    if (currently_loading())
         return;
 
     forget_image(&previous_image);
@@ -530,7 +605,7 @@ void reload_changed_files(gpointer * stat_data)
     if (DIFFERENT(next))
         forget_image(&next_image);
 
-    if (DIFFERENT(current))
+    if (now->current.present && DIFFERENT(current))
         reload_current_image();
 
     g_free(before);
@@ -577,7 +652,6 @@ static gboolean check_direction(const gchar * filename, gint dir)
      * or just after the next one.
      */
     if (next_node != NULL && filename == next_node->data) {
-        gint number;
         GlivImage *new;
 
         if (*prev != NULL)
@@ -585,16 +659,13 @@ static gboolean check_direction(const gchar * filename, gint dir)
         *prev = *next;
         *next = NULL;
 
-        number = current_image->number + 2 * dir;
-
-        new = load_file(filename);
+        new = load_file(filename, NULL);
         if (new == NULL) {
             remove_from_list(next_node);
             return TRUE;
         }
 
         new->node = next_node;
-        new->number = number;
 
         render_next_image(new);
         g_object_unref(new);
@@ -606,8 +677,6 @@ static gboolean check_direction(const gchar * filename, gint dir)
 
 gboolean menu_load(const gchar * filename)
 {
-    GList *node;
-    gint number = 0;
     GlivImage *im;
     static gboolean loading = FALSE;
 
@@ -618,37 +687,21 @@ gboolean menu_load(const gchar * filename)
 
     loading = TRUE;
 
-    if (is_loading() || filename == current_image->node->data ||
+    if (currently_loading() || filename == current_image->node->data ||
         check_direction(filename, -1) || check_direction(filename, 1)) {
         loading = FALSE;
         return FALSE;
     }
 
-    for (node = get_list_head(); node != NULL; node = node->next) {
-        if (filename == node->data)
-            break;
-
-        number++;
-    }
-
-    if (node == NULL) {
-        /* The image is not in the list anymore. */
-        loading = FALSE;
-        return FALSE;
-    }
-
-    im = load_file(filename);
+    im = load_file(filename, NULL);
     if (im == NULL) {
-        remove_from_list(node);
         loading = FALSE;
         return FALSE;
     }
 
+    im->node = find_node_by_name(filename);
     unload_images();
 
-    im->node = node;
-    im->number = number;
-
     render_next_image(im);
     loading = FALSE;
     g_object_unref(im);
@@ -672,15 +725,8 @@ void unload(GList * node)
 /* Called when we want only one image in memory. */
 void unload_images(void)
 {
-    if (previous_image != NULL) {
-        g_object_unref(previous_image);
-        previous_image = NULL;
-    }
-
-    if (next_image != NULL) {
-        g_object_unref(next_image);
-        next_image = NULL;
-    }
+    forget_image(&previous_image);
+    forget_image(&next_image);
 }
 
 /*** Slide show ***/
@@ -706,19 +752,19 @@ static void stop_slide_show(void)
 
 static gboolean slide_show_next(void)
 {
-    if (load_direction(slide_show_direction) == FALSE) {
-        /* No more images. */
-        stop_slide_show();
-    }
-
+    load_direction(slide_show_direction);
+    if (slide_show_started())
+        slide_show_advance();
     return FALSE;
 }
 
 static void slide_show_advance(void)
 {
     slide_show_remove_timer();
-    slide_show_timer = g_timeout_add(options->duration * 1000,
-                                     (GSourceFunc) slide_show_next, NULL);
+    slide_show_timer = g_timeout_add_full(G_PRIORITY_LOW,
+                                          options->duration * 1000,
+                                          (GSourceFunc) slide_show_next, NULL,
+                                          NULL);
 }
 
 void start_slide_show(void)
